<!-- 
This code was adapted from that described in the article PHP and XML: 
Parsing RSS 1.0 by Kevin Yank which was found at \
http://www.sitepoint.com/print/php-xml-parsing-rss-1-0

It reads an xml file, in particular an rss file which has <item> tags and 
within <item> tags has <title> tags, <description> tags and <link> tags. 
It only displays six items.  
 -->
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>XML Parsing</title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<style type="text/css">
<!--
body,td,th {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: medium;
}
body {
	background-color: #99CCFF;
}
-->
</style></head>

<body>
<h2>Library news</h2>
<?php  

	$insideitem = false;	//Boolean variable that determines if we are betweeb <item> tags
	$tag = "";				//used to hold tag data to determine what kind of xml tag we are dealing with
	$title = "";			//used to hold title information from xml file before it is displayed on the page
	$description = "";		//used to hold description information from xml file before it is displayed on the page
	$link = "";				//used to hold link information from xml file before it is displayed on the page
	$itemcount=0;			//used to count the number of items presented on the page -- limited to six


//***************************************************************************
// This function is used when we start looking at an element (a opening tag) 
// within the xml-rss file.  If we are inside an item, it just sets the $tag 
// variable.  If we are not inside a tag variable, it asks if tag we are 
// looking at is an item tag -- if so it sets the insideitem varaible to true 
// and increments the count
//***************************************************************************
function startElement($parser, $tagName, $attrs)
{
	global $insideitem, $tag, $title, $description, $link, $count;
	
	if($insideitem)
	{
		$tag=$tagName;
	}
	elseif($tagName=="ITEM")
	{
		$insideitem=true;
		$itemcount++;
	}
} //end of startElement function


//***************************************************************************
// This function is used when we finish looking at an element (a closing tag) 
// within the xml-rss file.  If we are finishing an item and the itemcount is 
// less than or equal to 6, then it prints the appropriate html code to the 
// page.  It also resets the $title, $link and $description variables back 
// to empty strings and sets the $insideitem boolean to false since we are 
// leaving an item 
//***************************************************************************
function endElement($parser, $tagName)
{
	global $insideitem, $tag, $title, $description, $link, $count;
	
	if($tagName=="ITEM" && $itemcount<=6)
	{
		printf("<p><b><a href= '%s'>%s</a></a></p>", 
			trim($link),htmlspecialchars(trim($title)));
		printf("<p>%s</p>",htmlspecialchars(trim($description)));
		
		/*
		The function printf() is a C-style print function.  Its first argument is 
		a string to be printed to the page.  The %s's seen within the first arguement 
		above indicate one is to insert string expressions in at that location.  The 
		expressions to be inserted are included as subsequent arguments of the 
		printf function. 
		
		The trim() function strips away any white space that might be found at the 
		beginning or end of the string argument. 
		
		The htmlspecialchars() function takes certain HTML code and converts it to 
		its "entity" version.  For example <b> is converted to &lt;b&gt; thus instead 
		of the text being bolded, one sees instead the bold tag. One could also use 
		the htmlentities() function.  Consider using the nl2br() function if you want 
		returns respected.  And consider using the stripslashes() function if quotes 
		are being replaced by slash quotes. 
		*/
		
		$title="";
		$description="";
		$link="";
		$insideitem=false;
	}
}

//***************************************************************************
// This function is used when we are not looking at tags but at data -- the 
// stuff between the tags.  If we are inside an item, then we ask what kind 
// of tag was most recently read (see startElement function).  We then 
// concatenate that data to the appropriate variable.  The parser function 
// capitalizes tag names, so the case statement uses capitalized tag names  
//***************************************************************************
function characterData($parser, $data)
{
	global $insideitem, $tag, $title, $description, $link, $count;
	
	if($insideitem)
	{
		switch($tag)
		{
			case "TITLE":
				$title .=$data;
				break;
			case "DESCRIPTION":
				$description .= $data;
				break;
			case "LINK":
				$link .= $data;
				break;
		}
	}
}

//the program starts here

	$xml_parser = xml_parser_create();
	//announce that you intend to use an XML parser and give it a name

	xml_set_element_handler($xml_parser,"startElement","endElement");
	/*
	The parser knows when it finds opening and closing tags, but since XML is user defined 
	you have to say what should happen when it comes across an opeing tag (done in the startElement 
	function) and what should happen when it comes across a closing tag (done in the endElement
	function) 
	*/

	xml_set_character_data_handler($xml_parser, "characterData");
	/*
	The parser knows when it is between opening and closing tags, but since XML is user defined 
	you have to say what should be done with the "data" (done in the chracterData function)
	*/

	$fp = fopen("librarynews.xml","r");
	/*
	The variable $fp is a file pointer.  It lets the program know where to find the file. 
	Here we have used a relative address indicating that the file should be found in the 
	same folder/directory as the program file. The "r" argument indicates that this file 
	is to be read. 
	*/

	while($data = fread($fp, 4096))
	{
		xml_parse($xml_parser,$data,feof($fp)) 
		or die(sprintf("XML error: %s at line %d", 
			xml_error_string(xml_get_error_code($xml_parser)),
			xml_get_current_line_number($xml_parser)));
	}
	/*
	Read data from the file in 4 kilobyte chunks.  The while loop says that while 
	there is still data to be read we should stay in the loop. 
	
	The xml_parse() function tells the parser we established to parse the 
	data just read in.  If the xml file was larger than 4KB, then certain closing 
	tags might not be found in the data taken in so far.  The feof($fp) seen 
 	as the third argument of the xml_parse() lets the parser know whether the 
	end-of-file has been reached.  If the end-of-file has been reached and the 
	appropriate closing has not been found, then there is a problem. 
	
	The xml_parse() function refers to the rules you establish in the 
	startElement, endElement and characterData functions. The or part is used 
	if the xml parsing goes wrong. 
	
	*/


	fclose($fp);		//close the "connection/stream" to the data file
	
	xml_parser_free($xml_parser);
	//in a longer program this would free up any resources being used by the parser
	//the freeing up happens at the end of the program anyway -- so this is a good 
	//practice but not really needed here
	

?>
</body>
</html>
